#!/bin/bash

MAJOR=0
MINOR=0
VER=0
LATEST_VER=0
UPDATE_VER=0
CURR_APP=""
PREV_APP=""
PREV_UPDATE_APP=""
NEW_APP=""
DEL_APP=""

echo "**INFO: starting update.."

#store the serial number in u-boot
fw_setenv serialnumber `cat /serialnumber`

# start in standby feature
enableamp="if printenv startInStandby; then; else i2c mw 0x18 0x01 0x01; fi;"
fw_setenv enable_amp $enableamp

ls -l /sentinel_*[0-9]_*[0-9] > /dev/null
if [ $? -gt 0 ]; then
	echo "**WARNING: currently no app on device!"
else
	for i in /sentinel_*[0-9]_*[0-9]; do
		CURR_APP=$i

		if [ -f "$DEL_APP" ]; then
               		echo "**INFO: removing old app: $DEL_APP"
               		rm $DEL_APP
        	fi

		MAJOR=${i#*_}
		MAJOR=${MAJOR%_*}
		let MAJOR=$MAJOR*100
		MINOR=${i#*_}
		MINOR=${MINOR#*_}
		let VER=$MAJOR+$MINOR
		if [ $VER -gt $LATEST_VER ]; then
			LATEST_VER=$VER
			if [ -f "$PREV_APP" ]; then
				echo "**INFO: removing old app: $PREV_APP"
				rm $PREV_APP 
			fi
			PREV_APP=$CURR_APP
			echo "**INFO: app current version = $LATEST_VER, $CURR_APP"
		else
			DEL_APP=$CURR_APP
		fi
	done
fi

for i in /tmp/updateFrontFw/sentinel_*[0-9]_*[0-9]; do
        NEW_APP=$i
        MAJOR=${i#*_}
        MAJOR=${MAJOR%_*}
        let MAJOR=$MAJOR*100
        MINOR=${i#*_}
        MINOR=${MINOR#*_}
        let VER=$MAJOR+$MINOR
        if [ $VER -gt $UPDATE_VER ]; then
                UPDATE_VER=$VER
                if [ -f "$PREV_UPDATE_APP" ]; then
                        rm $PREV_UPDATE_APP
                fi
                PREV_UPDATE_APP=$NEW_APP
                echo "**INFO: app update versie = $UPDATE_VER, $NEW_APP"
        else
                if [ -f "$NEW_APP" ]; then
                        rm $NEW_APP
                fi
        fi
done

NEEDS_REBOOT=0

if [ $UPDATE_VER -gt $LATEST_VER -o -f /tmp/updateFrontFw/forceVersion ]; then
	echo "**INFO: replacing app $LATEST_VER with app $UPDATE_VER"
	
	if [ -f /etc/Toradex ]; then
		SPLASH=reloading_toradex.rw
	else
		kill -9 `pidof /bin/sh watchdog_script` || echo "**WARNING: failed killing watchdog script!"
		kill -9 `pidof sentinel*` || echo "**WARNING: failed killing front app!"
		SPLASH=reloading_cm-x300.rw
	fi

	sync
	echo "**INFO: removing old app: $CURR_APP"
	rm $CURR_APP
	cat /tmp/updateFrontFw/$SPLASH > /dev/fb0

	NEW_APP=${NEW_APP#/tmp/updateFrontFw/}

	if [ -f /etc/Toradex ]; then
		#toradex
		echo "**INFO: copying new Toradex app: $NEW_APP"
		cp /tmp/updateFrontFw/Toradex/$NEW_APP /$NEW_APP
		echo "**INFO: copying email app.."
		cp /tmp/updateFrontFw/Toradex/msmtp /usr/bin/
		cp /tmp/updateFrontFw/Toradex/libcrypto.so.1.1 /usr/lib/
		cp /tmp/updateFrontFw/Toradex/libssl.so.1.1 /usr/lib/
	else
		#cm-x300
		cp /tmp/updateFrontFw/autoprofile /etc/init.d/autoprofile
		cp /tmp/updateFrontFw/watchdog_script /watchdog_script
		cp /tmp/updateFrontFw/fstab /etc/fstab
		cp /tmp/updateFrontFw/start_ssh.sh /etc/init.d/start_ssh.sh
		echo "**INFO: copying new cm-x300 app: $NEW_APP"
		cp /tmp/updateFrontFw/$NEW_APP /$NEW_APP
		echo "**INFO: copying email app.."
		cp /tmp/updateFrontFw/msmtp /usr/bin/
		cp /tmp/updateFrontFw/libcrypto.so.1.0.0 /usr/lib/
		cp /tmp/updateFrontFw/libssl.so.1.0.0 /usr/lib/
		cp /tmp/updateFrontFw/ca-certificates.crt /etc/ssl/certs/
	fi

	let a=$UPDATE_VER/100
	let b=$UPDATE_VER%100
	echo "front version: $a.$b" > /etc/frontversion
	sync

	rm -f /sentinel	
	ln -s $NEW_APP /sentinel

	echo "rebooted" > /tmp/systemRebooted
	NEEDS_REBOOT=1
else
	echo "**INFO: app up-to-date, only copying limits file.."
fi

cp /tmp/updateFrontFw/protectionLimits.txt /protectionLimits.txt

rm -rf /tmp/updateFrontFw

#fix for flickering displays on cm-x300
if [ ! -f /etc/Toradex ] ; then
	echo "**INFO: applying flicker patch.."
	nandboot="ubi.mtd=fs,2048 root=ubi0:ubi-image rootfstype=ubifs console=ttyS2,38400"
	linuxlcd="video=pxafb:vmem:2M,mode:480x272-16,left:3,right:2,upper:2,lower:3,hsynclen:42,vsynclen:11,hsync:0,vsync:0,outputen:0,pixclockpol:0,pixclock:100000"
	fw_setenv bootargs $nandboot $linuxlcd quiet
fi

sync
sync

if [ $NEEDS_REBOOT -eq 1 ]; then
	if [ -f /etc/Toradex ]; then
		echo "**INFO: Toradex restart app"
		systemctl restart sentinel.service
	else
		echo "**INFO: cm-x300 reboots"
		/sbin/reboot
	fi
fi

